# Seminar 06 – Demo – Alza.cz API

In today's demo, we will create a simple API for managing products in our e-shop. We will use the **Express** web application framework along with the **Redis** database. Finally, we explore how to validate incoming requests using the **Zod** library.

## Preparation

### Redis

We will employ Redis as our database solution. No need to fret, as all the necessary operations have already been implemented for us in `/src/products/repository.ts`. To get started, let's run the Docker container for Redis. Execute the following command:

```sh
docker run -d --name redis -p 6379:6379 redis
```

### .env

The application relies on specific environment variables. To ensure they are loaded correctly, place them in the `.env` file located in the root directory of the project. Copying the default values from `.env.example` should be enough.

Now that everything is set up and prepared, we're ready to dive into our tasks. Let's get to work!

## Task 1: Check out the documentation

In the `/api-documentation` directory, you'll find the documentation for the API we are about to develop. We want to get familiar with it first so that we can then create the required API based on it. To do this, let's start our development server by running the `npm start` command in the terminal (after installing the dependencies, of course 😉).

Once the server is up and running, we can access the Swagger UI at [http://localhost:6001/api-documentation](http://localhost:6001/api-documentation).

## Task 2: Check out the repository

The file `/src/products/repository.ts` contains a repository that handles various operations related to products in the database. We'll utilize the functions provided by this repository to implement the operations described in the documentation.

So let's take some time to familiarize ourselves with these functions along with the related type definitions.

## Task 3: Create a router

Most applications allows working with multiple resources or entities. To keep the related controllers separated, we want to create a Router for each resource. Routers are modular, mountable route handlers that, once registered in the Express instance, are responsible for processing incoming requests to a specific route.

1. Go to the `/src/products/router.ts` file and create a new router for all URL addresses starting with `/products`.

2. Expose the router instance and register it in the `app` created in `/src/index.ts`.

3. Let's try to implement a simple handler within the product router that returns any value for a GET request to `/products`. Try to call this endpoint by Swagger ("List products").

## Task 4: Create a controller

We want to create handlers for each endpoint in the controller located at `/src/products/controller.ts`. Let's move our handler there and then use it in the router.

Then prepare handlers for the remaining endpoints.

Try to get (and maybe dump to the console) data for creating a product (POST /products) and parameters for filtering the product list (GET /products).

## Task 5: Consume the repository

Import the `productRepository` in the controller and implement all four handlers using its functionality.

## Task 6: Repository errors handling

You probably noticed that the repository doesn't directly return data. Instead, it returns a Result object. This design enables a clearer control flow in the code without the need to catch exceptions within a try-catch block.

Now, we need to deal with situations when errors happen and respond with the right HTTP response and status code for each error.

## Task 7: Request runtime validation

As TypeScript doesn't check types at runtime, receiving incorrect data in a request can cause the application to crash. However, we want to handle such situations correctly by returning a "400 Bad Request" error.

To achieve this, we can utilize the `zod` library. It enables us to define schemas that validate objects at runtime, ensuring type safety alongside TypeScript types.

Our tasks:

1. Define validation schemas for each request in the file `src/products/validationSchemas.ts`.
2. Validate requests within the handlers in the product controller using the `safeParseAsync` function.
3. Handle errors properly.

## (Task 8)

Consider how we can refactor the code in the controller to minimize duplication in error handling. One potential solution is implemented in the provided solution.
